@@ -14,8 +14,8 @@
  * limitations under the License.
  */
 
-#include <ctype.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 
 #include <cmark.h>
@@ -295,30 +295,9 @@ CFCCHtml_write_html_docs(CFCCHtml *self) {
         char *filename = filenames[i];
         char *path     = CFCUtil_sprintf("%s" CHY_DIR_SEP "%s", doc_path,
                                          filename);
-
-        // Make path.
-
-        char *dir  = CFCUtil_strdup(path);
-        for (size_t j = strlen(dir); j--;) {
-            if (dir[j] == CHY_DIR_SEP_CHAR) {
-                do {
-                    dir[j] = '\0';
-                } while (j-- && dir[j] == CHY_DIR_SEP_CHAR);
-                break;
-            }
-        }
-
-        if (dir[0] != '\0' && !CFCUtil_is_dir(dir)) {
-            CFCUtil_make_path(dir);
-            if (!CFCUtil_is_dir(dir)) {
-                CFCUtil_die("Can't make path %s", dir);
-            }
-        }
-
         char *html_doc = html_docs[i];
         CFCUtil_write_if_changed(path, html_doc, strlen(html_doc));
         FREEMEM(html_doc);
-        FREEMEM(dir);
         FREEMEM(path);
         FREEMEM(filename);
     }
@@ -747,7 +726,7 @@ S_html_create_methods(CFCClass *klass) {
  */
 static char*
 S_html_create_fresh_methods(CFCClass *klass, CFCClass *ancestor) {
-    CFCMethod  **fresh_methods = CFCClass_fresh_methods(klass);
+    CFCMethod  **fresh_methods = CFCClass_fresh_methods(ancestor);
     const char  *prefix        = CFCClass_get_prefix(klass);
     char        *result        = CFCUtil_strdup("");
 
@@ -757,7 +736,14 @@ S_html_create_fresh_methods(CFCClass *klass, CFCClass *ancestor) {
             continue;
         }
 
-        if (!CFCMethod_is_fresh(method, ancestor)) {
+        const char *name = CFCMethod_get_name(method);
+        if (strcmp(name, "Destroy") == 0) {
+            // Destroy must not be called directly.
+            continue;
+        }
+
+        CFCMethod *other = CFCClass_method(klass, name);
+        if (!CFCMethod_is_fresh(other, ancestor)) {
             // The method is implementated in a subclass and already
             // documented.
             continue;
@@ -767,7 +753,6 @@ S_html_create_fresh_methods(CFCClass *klass, CFCClass *ancestor) {
             result = CFCUtil_cat(result, "<dl>\n", NULL);
         }
 
-        const char *name = CFCMethod_get_name(method);
         result = CFCUtil_cat(result, "<dt id=\"func_", name, "\">",
                              name, NULL);
         if (CFCMethod_abstract(method)) {
@@ -1091,8 +1076,8 @@ S_type_to_html(CFCType *type, const char *sep, CFCClass *doc_class) {
             CFCUtil_die("Unprefixed object specifier '%s'", specifier);
         }
 
-        size_t      offset     = underscore + 1 - specifier;
-        char       *prefix     = CFCUtil_strndup(specifier, offset);
+        ptrdiff_t   offset     = underscore + 1 - specifier;
+        char       *prefix     = CFCUtil_strndup(specifier, (size_t)offset);
         const char *struct_sym = specifier + offset;
 
         if (!klass) {
@@ -1117,7 +1102,7 @@ S_type_to_html(CFCType *type, const char *sep, CFCClass *doc_class) {
     const char *const_str = CFCType_const(type) ? "const " : "";
 
     int indirection = CFCType_get_indirection(type);
-    size_t asterisk_offset = indirection < 10 ? 10 - indirection : 0;
+    ptrdiff_t asterisk_offset = indirection < 10 ? 10 - indirection : 0;
     const char *asterisks = "**********";
     const char *ind_str   = asterisks + asterisk_offset;
 
@@ -1203,7 +1188,7 @@ S_relative_url(const char *url, CFCClass *base, int dir_level) {
     }
 
     // Create path back to root
-    size_t bytes = dir_level * 3;
+    size_t bytes = (size_t)(dir_level * 3);
     char *prefix = (char*)MALLOCATE(bytes + 1);
     for (size_t i = 0; i < bytes; i += 3) {
         memcpy(prefix + i, "../", 3);
