@@ -16,7 +16,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <ctype.h>
 
 #ifndef true
   #define true 1
@@ -34,13 +33,9 @@
 #include "CFCUtil.h"
 #include "CFCVariable.h"
 #include "CFCFileSpec.h"
+#include "CFCJson.h"
 
-typedef struct CFCClassRegEntry {
-    char *key;
-    struct CFCClass *klass;
-} CFCClassRegEntry;
-
-static CFCClassRegEntry *registry = NULL;
+static CFCClass **registry = NULL;
 static size_t registry_size = 0;
 static size_t registry_cap  = 0;
 
@@ -139,22 +134,22 @@ CFCClass_validate_class_name(const char *class_name) {
     for (;;substring++) {
         if (*substring == 0)          { return false; }
         else if (*substring == ':')   { return false; }
-        else if (islower(*substring)) { break; }
+        else if (CFCUtil_islower(*substring)) { break; }
     }
 
     // Must be UpperCamelCase, separated by "::".
     const char *ptr = class_name;
-    if (!isupper(*ptr)) { return false; }
+    if (!CFCUtil_isupper(*ptr)) { return false; }
     while (*ptr != 0) {
         if (*ptr == 0) { break; }
         else if (*ptr == ':') {
             ptr++;
             if (*ptr != ':') { return false; }
             ptr++;
-            if (!isupper(*ptr)) { return false; }
+            if (!CFCUtil_isupper(*ptr)) { return false; }
             ptr++;
         }
-        else if (!isalnum(*ptr)) { return false; }
+        else if (!CFCUtil_isalnum(*ptr)) { return false; }
         else { ptr++; }
     }
 
@@ -178,7 +173,7 @@ S_validate_nickname(const char *nickname) {
             if (strlen(nickname)) { return true; }
             else { break; }
         }
-        else if (!isupper(*ptr)) { break; }
+        else if (!CFCUtil_isupper(*ptr)) { break; }
     }
 
     // Same as one component of a class name.
@@ -274,7 +269,7 @@ CFCClass_do_create(CFCClass *self, struct CFCParcel *parcel,
     char *short_class_var = (char*)MALLOCATE(struct_sym_len + 1);
     size_t i;
     for (i = 0; i < struct_sym_len; i++) {
-        short_class_var[i] = toupper(struct_sym[i]);
+        short_class_var[i] = CFCUtil_toupper(struct_sym[i]);
     }
     short_class_var[struct_sym_len] = '\0';
     self->short_class_var = short_class_var;
@@ -301,20 +296,20 @@ CFCClass_do_create(CFCClass *self, struct CFCParcel *parcel,
                     name, CFCParcel_get_name(parcel));
     }
 
-    // Skip class if it's from an include dir and the parcel was already
-    // processed in another source or include dir.
-    const char *class_source_dir  = CFCClass_get_source_dir(self);
-    const char *parcel_source_dir = CFCParcel_get_source_dir(parcel);
-    if (!CFCClass_included(self)
-        || !class_source_dir
-        || !parcel_source_dir
-        || strcmp(class_source_dir, parcel_source_dir) == 0
-    ) {
+    char *error;
+
+    CFCUTIL_TRY {
         // Store in registry.
         S_register(self);
-
-        CFCParcel_add_struct_sym(parcel, self->struct_sym);
     }
+    CFCUTIL_CATCH(error);
+
+    if (error) {
+        CFCBase_decref((CFCBase*)self);
+        CFCUtil_rethrow(error);
+    }
+
+    CFCParcel_add_struct_sym(parcel, self->struct_sym);
 
     return self;
 }
@@ -364,77 +359,64 @@ static void
 S_register(CFCClass *self) {
     if (registry_size == registry_cap) {
         size_t new_cap = registry_cap + 10;
-        registry = (CFCClassRegEntry*)REALLOCATE(
+        registry = (CFCClass**)REALLOCATE(
                        registry,
-                       (new_cap + 1) * sizeof(CFCClassRegEntry));
+                       (new_cap + 1) * sizeof(CFCClass*));
         for (size_t i = registry_cap; i <= new_cap; i++) {
-            registry[i].key = NULL;
-            registry[i].klass = NULL;
+            registry[i] = NULL;
         }
         registry_cap = new_cap;
     }
 
-    const char *prefix   = CFCParcel_get_prefix(self->parcel);
-    const char *name     = self->name;
-    const char *nickname = self->nickname;
-    const char *key      = self->full_struct_sym;
+    const char *prefix     = CFCParcel_get_prefix(self->parcel);
+    const char *name       = self->name;
+    const char *nickname   = self->nickname;
+    const char *struct_sym = self->full_struct_sym;
 
     for (size_t i = 0; i < registry_size; i++) {
-        CFCClass   *other          = registry[i].klass;
-        const char *other_prefix   = CFCParcel_get_prefix(other->parcel);
-        const char *other_name     = other->name;
-        const char *other_nickname = other->nickname;
+        CFCClass   *other        = registry[i];
+        const char *other_prefix = CFCParcel_get_prefix(other->parcel);
 
-        if (strcmp(name, other_name) == 0) {
+        if (strcmp(name, other->name) == 0) {
             CFCUtil_die("Two classes with name %s", name);
         }
-        if (strcmp(registry[i].key, key) == 0) {
+        if (strcmp(struct_sym, other->full_struct_sym) == 0) {
             CFCUtil_die("Class name conflict between %s and %s",
-                        name, other_name);
+                        name, other->name);
         }
         if (strcmp(prefix, other_prefix) == 0
-            && strcmp(nickname, other_nickname) == 0
+            && strcmp(nickname, other->nickname) == 0
            ) {
             CFCUtil_die("Class nickname conflict between %s and %s",
-                        name, other_name);
+                        name, other->name);
         }
     }
 
-    registry[registry_size].key   = CFCUtil_strdup(key);
-    registry[registry_size].klass = (CFCClass*)CFCBase_incref((CFCBase*)self);
+    registry[registry_size] = (CFCClass*)CFCBase_incref((CFCBase*)self);
     registry_size++;
 }
 
 #define MAX_SINGLETON_LEN 256
 
 CFCClass*
-CFCClass_fetch_singleton(CFCParcel *parcel, const char *class_name) {
+CFCClass_fetch_singleton(const char *class_name) {
     CFCUTIL_NULL_CHECK(class_name);
 
-    // Build up the key.
-    const char *last_colon = strrchr(class_name, ':');
-    const char *struct_sym = last_colon
-                             ? last_colon + 1
-                             : class_name;
-    const char *prefix = parcel ? CFCParcel_get_prefix(parcel) : "";
-    size_t prefix_len = strlen(prefix);
-    size_t struct_sym_len = strlen(struct_sym);
-    if (prefix_len + struct_sym_len > MAX_SINGLETON_LEN) {
-        CFCUtil_die("names too long: '%s', '%s'", prefix, struct_sym);
+    for (size_t i = 0; i < registry_size; i++) {
+        if (strcmp(registry[i]->name, class_name) == 0) {
+            return registry[i];
+        }
     }
-    char key[MAX_SINGLETON_LEN + 1];
-    sprintf(key, "%s%s", prefix, struct_sym);
-
-    return CFCClass_fetch_by_struct_sym(key);
+    return NULL;
 }
 
 CFCClass*
-CFCClass_fetch_by_struct_sym(const char *key) {
-    CFCUTIL_NULL_CHECK(key);
+CFCClass_fetch_by_struct_sym(const char *struct_sym) {
+    CFCUTIL_NULL_CHECK(struct_sym);
 
     for (size_t i = 0; i < registry_size; i++) {
-        if (strcmp(registry[i].key, key) == 0) {
-            return registry[i].klass;
+        if (strcmp(registry[i]->full_struct_sym, struct_sym) == 0) {
+            return registry[i];
         }
     }
     return NULL;
@@ -443,14 +425,13 @@ CFCClass_fetch_by_struct_sym(const char *key) {
 void
 CFCClass_clear_registry(void) {
     for (size_t i = 0; i < registry_size; i++) {
-        CFCClass *klass = registry[i].klass;
+        CFCClass *klass = registry[i];
         if (klass->parent) {
             // Break circular ref.
             CFCBase_decref((CFCBase*)klass->parent);
             klass->parent = NULL;
         }
         CFCBase_decref((CFCBase*)klass);
-        FREEMEM(registry[i].key);
     }
     FREEMEM(registry);
     registry_size = 0;
@@ -464,6 +445,9 @@ CFCClass_add_child(CFCClass *self, CFCClass *child) {
     if (self->tree_grown) {
         CFCUtil_die("Can't call add_child after grow_tree");
     }
+    if (self->is_final) {
+        CFCUtil_die("Can't inherit from final class %s", self->name);
+    }
     if (self->is_inert) {
         CFCUtil_die("Can't inherit from inert class %s", self->name);
     }
@@ -758,6 +742,22 @@ CFCClass_tree_to_ladder(CFCClass *self) {
     return ladder;
 }
 
+void
+CFCClass_read_host_data_json(CFCClass *self, CFCJson *hash, const char *path) {
+    CFCJson *method_hash = CFCJson_find_hash_elem(hash, "methods");
+    if (!method_hash) { return; }
+
+    CFCJson **children = CFCJson_get_children(method_hash);
+    for (int i = 0; children[i]; i += 2) {
+        const char *method_name = CFCJson_get_string(children[i]);
+        CFCMethod *method = CFCClass_method(self, method_name);
+        if (!method) {
+            CFCUtil_die("Method '%s' in '%s' not found", method_name, path);
+        }
+        CFCMethod_read_host_data_json(method, children[i+1], path);
+    }
+}
+
 CFCMethod**
 CFCClass_fresh_methods(CFCClass *self) {
     return self->fresh_methods;
