@@ -15,7 +15,6 @@
  */
 
 #include <string.h>
-#include <ctype.h>
 
 #include "CFCPyMethod.h"
 #include "CFCPyTypeMap.h"
@@ -276,8 +275,8 @@ S_build_pymeth_invocation(CFCMethod *method) {
         if (strlen(ret_type_str) > 63) {
             CFCUtil_die("Unexpectedly long type name: %s", ret_type_str);
         }
-        for (int i = 0, max = strlen(ret_type_str) + 1; i < max; i++) {
-            type_upcase[i] = toupper(ret_type_str[i]);
+        for (size_t i = 0, max = strlen(ret_type_str) + 1; i < max; i++) {
+            type_upcase[i] = CFCUtil_toupper(ret_type_str[i]);
         }
         const char pattern[] =
             "    %s cfcb_RESULT = CALL_PYMETH_%s((PyObject*)self, \"%s\", cfcb_ARGS);";
@@ -435,7 +434,7 @@ S_gen_arg_increfs(CFCParamList *param_list, int first_tick) {
     CFCVariable **vars = CFCParamList_get_variables(param_list);
     int num_vars = CFCParamList_num_vars(param_list);
     char *content = CFCUtil_strdup("");
-    for (int i = first_tick;i < num_vars; i++) {
+    for (int i = first_tick; i < num_vars; i++) {
         CFCType *type = CFCVariable_get_type(vars[i]);
         if (CFCType_decremented(type)) {
             const char *name = CFCVariable_get_name(vars[i]);
@@ -515,7 +514,7 @@ S_gen_meth_invocation(CFCMethod *method, CFCClass *invoker) {
         maybe_assign = "";
     }
     else {
-        maybe_declare = CFCUtil_sprintf("    %s retvalCF;\n",
+        maybe_declare = CFCUtil_sprintf("    %s retvalCF = 0;\n",
                                         CFCType_to_c(return_type));
         maybe_assign = "retvalCF = ";
     }
@@ -646,7 +645,7 @@ CFCPyMethod_pymethoddef(CFCMethod *method, CFCClass *invoker) {
     char *meth_sym = CFCMethod_full_method_sym(method, invoker);
     char *micro_sym = CFCUtil_strdup(CFCSymbol_get_name((CFCSymbol*)method));
     for (int i = 0; micro_sym[i] != 0; i++) {
-        micro_sym[i] = tolower(micro_sym[i]);
+        micro_sym[i] = CFCUtil_tolower(micro_sym[i]);
     }
 
     char pattern[] =
