@@ -14,6 +14,8 @@
  * limitations under the License.
  */
 
+#include <string.h>
+
 #include "charmony.h"
 
 #define CFC_USE_TEST_MACROS
@@ -37,18 +39,22 @@ static void
 S_run_prereq_tests(CFCTest *test);
 
 static void
-S_run_parcel_tests(CFCTest *test);
+S_run_basic_tests(CFCTest *test);
+
+static void
+S_run_extended_tests(CFCTest *test);
 
 const CFCTestBatch CFCTEST_BATCH_PARCEL = {
     "Clownfish::CFC::Model::Parcel",
-    29,
+    41,
     S_run_tests
 };
 
 static void
 S_run_tests(CFCTest *test) {
     S_run_prereq_tests(test);
-    S_run_parcel_tests(test);
+    S_run_basic_tests(test);
+    S_run_extended_tests(test);
 }
 
 static void
@@ -77,22 +83,95 @@ S_run_prereq_tests(CFCTest *test) {
 }
 
 static void
-S_run_parcel_tests(CFCTest *test) {
+S_run_basic_tests(CFCTest *test) {
+    CFCVersion *version = CFCVersion_new("v32.10.102");
+    CFCVersion *major   = CFCVersion_new("v32.0.0");
+    CFCParcel *foo = CFCParcel_new("Foo", "FooNick", version, major, NULL);
+    OK(test, foo != NULL, "new");
+    STR_EQ(test, CFCParcel_get_name(foo), "Foo", "get_name");
+    STR_EQ(test, CFCParcel_get_nickname(foo), "FooNick", "get_nickname");
+    STR_EQ(test, CFCVersion_get_vstring(CFCParcel_get_version(foo)),
+           "v32.10.102", "get_version");
+    STR_EQ(test, CFCVersion_get_vstring(CFCParcel_get_major_version(foo)),
+           "v32.0.0", "get_major_version");
+    OK(test, !CFCParcel_included(foo), "not included");
+    OK(test, !CFCParcel_is_installed(foo), "not installed");
+    CFCParcel_register(foo);
+
     {
-        CFCParcel *parcel = CFCParcel_new("Foo", NULL, NULL, NULL);
-        OK(test, parcel != NULL, "new");
-        OK(test, !CFCParcel_included(parcel), "not included");
-        CFCBase_decref((CFCBase*)parcel);
+        CFCParcel *same_name = CFCParcel_new("Foo", NULL, NULL, NULL, NULL);
+        char      *error;
+
+        CFCUTIL_TRY {
+            CFCParcel_register(same_name);
+        }
+        CFCUTIL_CATCH(error);
+        OK(test, error && strstr(error, "already registered"),
+           "can't register two parcels with the same name");
+
+        FREEMEM(error);
+        CFCBase_decref((CFCBase*)same_name);
     }
 
     {
-        CFCFileSpec *file_spec = CFCFileSpec_new(".", "Parcel", true);
-        CFCParcel *parcel = CFCParcel_new("Foo", NULL, NULL, file_spec);
-        OK(test, CFCParcel_included(parcel), "included");
-        CFCBase_decref((CFCBase*)parcel);
-        CFCBase_decref((CFCBase*)file_spec);
+        CFCParcel *same_nick
+            = CFCParcel_new("OtherFoo", "FooNick", NULL, NULL, NULL);
+        char *error;
+
+        CFCUTIL_TRY {
+            CFCParcel_register(same_nick);
+        }
+        CFCUTIL_CATCH(error);
+        OK(test, error && strstr(error, "already registered"),
+           "can't register two parcels with the same nickname");
+
+        FREEMEM(error);
+        CFCBase_decref((CFCBase*)same_nick);
     }
 
+    CFCFileSpec *file_spec = CFCFileSpec_new(".", "Parcel", ".cfp", true);
+    CFCParcel *included_foo
+        = CFCParcel_new("IncludedFoo", NULL, NULL, NULL, file_spec);
+    OK(test, CFCParcel_included(included_foo), "included");
+    STR_EQ(test, CFCParcel_get_cfp_path(included_foo),
+           "." CHY_DIR_SEP "Parcel.cfp", "get_cfp_path");
+    STR_EQ(test, CFCVersion_get_vstring(CFCParcel_get_version(included_foo)),
+           "v0", "version defaults to v0");
+    STR_EQ(test,
+           CFCVersion_get_vstring(CFCParcel_get_major_version(included_foo)),
+           "v0", "major_version defaults to v0");
+    CFCParcel_register(included_foo);
+
+    {
+        CFCParcel **all_parcels = CFCParcel_all_parcels();
+        OK(test, all_parcels[0] && all_parcels[1] && !all_parcels[2],
+           "all_parcels returns two parcels");
+        STR_EQ(test, CFCParcel_get_name(all_parcels[0]), "Foo",
+               "all_parcels returns parcel Foo");
+        STR_EQ(test, CFCParcel_get_name(all_parcels[1]), "IncludedFoo",
+               "all_parcels returns parcel IncludedFoo");
+    }
+
+    {
+        CFCParcel_add_inherited_parcel(foo, included_foo);
+        CFCParcel **inh_parcels = CFCParcel_inherited_parcels(foo);
+        OK(test, inh_parcels[0] && !inh_parcels[1],
+           "inherited_parcels returns one parcel");
+        STR_EQ(test, CFCParcel_get_name(inh_parcels[0]), "IncludedFoo",
+               "inh_parcels returns parcel IncludedFoo");
+        FREEMEM(inh_parcels);
+    }
+
+    CFCBase_decref((CFCBase*)included_foo);
+    CFCBase_decref((CFCBase*)file_spec);
+    CFCBase_decref((CFCBase*)foo);
+    CFCBase_decref((CFCBase*)major);
+    CFCBase_decref((CFCBase*)version);
+    CFCParcel_reap_singletons();
+}
+
+static void
+S_run_extended_tests(CFCTest *test) {
     {
         const char *json =
             "        {\n"
@@ -106,14 +185,18 @@ S_run_parcel_tests(CFCTest *test) {
     }
 
     {
-        const char *path = "t" CHY_DIR_SEP "cfbase" CHY_DIR_SEP "Animal.cfp";
-        CFCParcel *parcel = CFCParcel_new_from_file(path, NULL);
+        char *dir = CFCTest_path("cfbase");
+        CFCFileSpec *file_spec = CFCFileSpec_new(dir, "Animal", ".cfp", false);
+        CFCParcel *parcel = CFCParcel_new_from_file(file_spec);
         OK(test, parcel != NULL, "new_from_file");
         CFCBase_decref((CFCBase*)parcel);
+        CFCBase_decref((CFCBase*)file_spec);
+        FREEMEM(dir);
     }
 
     {
-        CFCParcel *parcel = CFCParcel_new("Crustacean", "Crust", NULL, NULL);
+        CFCParcel *parcel = CFCParcel_new("Crustacean", "Crust", NULL, NULL,
+                                          NULL);
         CFCParcel_register(parcel);
         STR_EQ(test, CFCVersion_get_vstring(CFCParcel_get_version(parcel)),
                "v0", "get_version");
@@ -163,15 +246,15 @@ S_run_parcel_tests(CFCTest *test) {
     }
 
     {
-        CFCFileSpec *foo_file_spec = CFCFileSpec_new(".", "Foo", true);
-        CFCParcel *foo = CFCParcel_new("Foo", NULL, NULL, foo_file_spec);
+        CFCFileSpec *foo_file_spec = CFCFileSpec_new(".", "Foo", ".cfp", true);
+        CFCParcel *foo = CFCParcel_new("Foo", NULL, NULL, NULL, foo_file_spec);
         CFCParcel_register(foo);
 
         CFCVersion *cfish_version = CFCVersion_new("v0.8.7");
         CFCFileSpec *cfish_file_spec
-            = CFCFileSpec_new(".", "Clownfish", true);
-        CFCParcel *cfish
-            = CFCParcel_new("Clownfish", NULL, cfish_version, cfish_file_spec);
+            = CFCFileSpec_new(".", "Clownfish", ".cfp", true);
+        CFCParcel *cfish = CFCParcel_new("Clownfish", NULL, cfish_version,
+                                         NULL, cfish_file_spec);
         CFCParcel_register(cfish);
 
         const char *crust_json =
@@ -185,11 +268,6 @@ S_run_parcel_tests(CFCTest *test) {
         CFCParcel *crust = CFCParcel_new_from_json(crust_json, NULL);
         CFCParcel_register(crust);
 
-        CFCParcel_check_prereqs(crust);
-        INT_EQ(test, CFCParcel_required(foo), false, "parcel not required");
-        INT_EQ(test, CFCParcel_required(cfish), true, "prereq required");
-        INT_EQ(test, CFCParcel_required(crust), true, "self required");
-
         CFCParcel **prereq_parcels = CFCParcel_prereq_parcels(crust);
         OK(test, prereq_parcels[0] != NULL, "prereq_parcels[0]");
         const char *name = CFCParcel_get_name(prereq_parcels[0]);
