@@ -14,6 +14,8 @@
  * limitations under the License.
  */
 
+#include <string.h>
+
 #define CFC_USE_TEST_MACROS
 #include "CFCBase.h"
 #include "CFCClass.h"
@@ -60,7 +62,7 @@ S_run_composite_tests(CFCTest *test);
 
 const CFCTestBatch CFCTEST_BATCH_TYPE = {
     "Clownfish::CFC::Model::Type",
-    360,
+    369,
     S_run_tests
 };
 
@@ -79,7 +81,7 @@ S_run_tests(CFCTest *test) {
 
 static void
 S_run_basic_tests(CFCTest *test) {
-    CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL);
+    CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL, NULL);
     CFCParcel_register(neato_parcel);
     CFCType *type = CFCType_new(0, neato_parcel, "mytype_t", 0);
 
@@ -112,7 +114,7 @@ S_run_basic_tests(CFCTest *test) {
 
 static void
 S_run_primitive_tests(CFCTest *test) {
-    CFCParcel *parcel = CFCParcel_new("Parcel", NULL, NULL, NULL);
+    CFCParcel *parcel = CFCParcel_new("Parcel", NULL, NULL, NULL, NULL);
     CFCType *type = CFCType_new(CFCTYPE_PRIMITIVE, parcel, "hump_t", 0);
     OK(test, CFCType_is_primitive(type), "is_primitive");
 
@@ -260,6 +262,20 @@ S_run_void_tests(CFCTest *test) {
     CFCBase_decref((CFCBase*)parser);
 }
 
+static char*
+S_try_new_object(CFCParcel *parcel, const char *specifier, int indirection) {
+    CFCType *type = NULL;
+    char    *error;
+
+    CFCUTIL_TRY {
+        type = CFCType_new_object(0, parcel, specifier, indirection);
+    }
+    CFCUTIL_CATCH(error);
+
+    CFCBase_decref((CFCBase*)type);
+    return error;
+}
+
 static void
 S_run_object_tests(CFCTest *test) {
     static const char *modifiers[4] = {
@@ -328,13 +344,40 @@ S_run_object_tests(CFCTest *test) {
         CFCBase_decref((CFCBase*)parser);
     }
 
-    CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL);
+    CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL, NULL);
     CFCClass *foo_class
         = CFCClass_create(neato_parcel, NULL, "Foo", NULL, NULL, NULL, NULL,
                           false, false, false);
     CFCType *foo = CFCType_new_object(0, neato_parcel, "Foo", 1);
     CFCType_resolve(foo);
 
+    {
+        static const char *bad_specifiers[5] = {
+            "foo", "Foo_Bar", "FOOBAR", "1Foo", "1FOO"
+        };
+        for (int i = 0; i < 5; i++) {
+            char *error = S_try_new_object(neato_parcel, bad_specifiers[i], 1);
+            OK(test, error && strstr(error, "specifier"),
+               "constructor rejects bad specifier");
+            FREEMEM(error);
+        }
+    }
+
+    {
+        char *error = S_try_new_object(neato_parcel, NULL, 1);
+        OK(test, error && strstr(error, "specifier"), "specifier required");
+        FREEMEM(error);
+    }
+
+    {
+        for (int indirection = 0; indirection <= 2; indirection += 2) {
+            char *error = S_try_new_object(neato_parcel, "Foo", indirection);
+            OK(test, error && strstr(error, "indirection"),
+               "invalid indirection of %d", indirection);
+            FREEMEM(error);
+        }
+    }
+
     {
         CFCType *another_foo = CFCType_new_object(0, neato_parcel, "Foo", 1);
         CFCType_resolve(another_foo);
@@ -356,7 +399,7 @@ S_run_object_tests(CFCTest *test) {
 
     {
         CFCParcel *foreign_parcel
-            = CFCParcel_new("Foreign", NULL, NULL, NULL);
+            = CFCParcel_new("Foreign", NULL, NULL, NULL, NULL);
         CFCClass *foreign_foo_class
             = CFCClass_create(foreign_parcel, NULL, "Foreign::Foo", NULL, NULL,
                               NULL, NULL, false, false, false);
@@ -424,7 +467,8 @@ S_run_va_list_tests(CFCTest *test) {
 static void
 S_run_arbitrary_tests(CFCTest *test) {
     {
-        CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL);
+        CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL,
+                                                NULL);
         CFCParcel_register(neato_parcel);
 
         CFCType *foo = CFCType_new_arbitrary(neato_parcel, "foo_t");
@@ -464,7 +508,7 @@ S_run_arbitrary_tests(CFCTest *test) {
 static void
 S_run_composite_tests(CFCTest *test) {
     CFCParser *parser = CFCParser_new();
-    CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL);
+    CFCParcel *neato_parcel = CFCParcel_new("Neato", NULL, NULL, NULL, NULL);
     CFCParser_set_parcel(parser, neato_parcel);
 
     {
@@ -493,6 +537,20 @@ S_run_composite_tests(CFCTest *test) {
         }
     }
 
+    {
+        CFCType *type = NULL;
+        char    *error;
+
+        CFCUTIL_TRY {
+            type = CFCType_new_composite(0, NULL, 0, NULL);
+        }
+        CFCUTIL_CATCH(error);
+        OK(test, error && strstr(error, "child"), "child required");
+
+        FREEMEM(error);
+        CFCBase_decref((CFCBase*)type);
+    }
+
     {
         CFCType *foo = CFCType_new_object(0, neato_parcel, "Foo", 1);
         CFCType *const_foo
